<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use HCM\Util\HCMParameters;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\FPA\Repository\LoteVinculoRepository;
use HCM\FPA\Services\LoteVinculoService;

class LoteVinculoController extends Crud {
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, Connection $connection, EntityManager $entityManager, LoteVinculoRepository $loteVinculoRepository, LoteVinculoService $loteVinculoService)
    {
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->loteVinculoRepository = $loteVinculoRepository;
        $this->loteVinculoService = $loteVinculoService;
        parent::__construct($dataSourceManager);
    }

    public function getLoteVinculo(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_lotevinc_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setLoteVinculo(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_lotevinc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeLoteVinculo = $this->loteVinculoRepository->existeLote($row['NRORG'], $row['NMLOTEVINC']);
                    
                if($existeLoteVinculo && ($existeLoteVinculo['NRLOTEVINC'] != $row['NRLOTEVINC'] || !isset($row['NRLOTEVINC'])) ) {
                    $response->addNotification(new Notification("Já existe um lote de vínculo com esse nome", Notification::TYPE_ALERT));
                    return;
                }  
               
                if($row['__is_new'] === true){
                    $row['NRLOTEVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_LOTEVINC', 1, 12, true); 
                }

                parent::save($request, $response);
            }  
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteLoteVinculo(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_lotevinc";
            $nrorg = $request->getDataSet()->getRows()[0]['NRORG'];
            $nrlotevinc = $request->getDataSet()->getRows()[0]['NRLOTEVINC'];

            $existeItens = $this->loteVinculoRepository->selectExisteItensFilhos($nrorg, $nrlotevinc);

            if(!$existeItens){
                parent::delete($request, $response);
            } else {
                $response->addNotification(new Notification("Existem itens relacionados a este registro, favor excluí-los.", Notification::TYPE_ALERT));
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getItemLoteVinculo(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_itemlotevinc_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRLOTEVINC', 'P_DTMESCOMPETENC');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setItemLoteVinculo(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_itemlotevinc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $vinculoDuplicado = false;
                foreach ($row['NRVINCULOM_LIST'] as $nrvinculom) {

                    $existeVinculoNoLote = $this->loteVinculoRepository->existeVinculoNoLote($row['NRORG'], $row['NRLOTEVINC'], $nrvinculom);
                    if ($existeVinculoNoLote) {
                        $msgRetorno[] = [
                            'NRVINCULOM'  => $nrvinculom,
                            'DSDESCRICAO'  => 'Vínculo já esta cadastrado no lote',
                            'IDTIPOMSG' =>  'ATENÇÃO'
                        ];
                        $vinculoDuplicado = true;
                        continue;
                    }

                    $row['NRVINCULOM'] = $nrvinculom;
                    $row['NRITEMLOTEVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITEMLOTEVINC', 1, 12, true); 
                    parent::save($request, $response);
                }

                if($vinculoDuplicado) {
                    $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('msgRetorno',  $msgRetorno)));
                    return;
                }
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteItemLoteVinculo(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_itemlotevinc";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function getAuditoria(){
        $auditoria = array();
        $auditoria['DTINCLUSAO'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
        $auditoria['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
        return $auditoria;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
} 
