<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\MnemonicoService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;

class MnemonicoController extends Crud {

    protected $dataSourceName = 'gpe_mnemonico';
    //protected $dataSourceNameget = 'getMnemonico';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MnemonicoService $mnemonicoservice){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->mnemonicoservice = $mnemonicoservice;
        parent::__construct($dataSourceManager);
    }

    public function getMnemonico(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setMnemonico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $mnemonicos =  $this->mnemonicoservice->getMnemonicoByCodigo($row['CDMNEMONICO']);
                foreach ($mnemonicos as $mnemonico) {
                    if ($row['__is_new'] || (!$row['__is_new'] && $mnemonico->getNrmnemonico() != $row['NRMNEMONICO'])) {
                        $response->addNotification(new DTO\Response\Notification('Mnemônico já existente', DTO\Response\Notification::TYPE_ERROR));
                        return;
                    }
                }
                
                $row['DSREPOSITORIO'] = isset($row['DSREPOSITORIO']) ? $row['DSREPOSITORIO'] : null;
                $row['DSENTIDADE'] = isset($row['DSENTIDADE']) ? $row['DSENTIDADE'] : null;
                $row['DSATRIBUTO'] = isset($row['DSATRIBUTO']) ? $row['DSATRIBUTO'] : null;
                
                if (array_key_exists('NRMNEMONICO', $row)) {
                    if ($row['NRMNEMONICO'] === '') {
                        $row['NRMNEMONICO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_MNEMONICO', 1, 12, true); 
                    }
                } else {
                    $row['NRMNEMONICO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_MNEMONICO', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteMnemonico(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 