<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2205Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getItensParaEnvioS2205($params) {
        try {
            $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,    
       NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC) as DTMESCOMPETENC,
       'A' as IDOPERACAO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'MONTH'),-24)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'MONTH'),-12)
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC) >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
 ORDER BY 1,3
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
                "P_DTREFNAOPERIOD" => isset($params['DTREFNAOPERIOD']) ? $params['DTREFNAOPERIOD'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosS2205($parametros) {
        try {
            $dataHistorico = \HCM\Util\DateUtil::getDataDeString($parametros['DTMESCOMPETENC'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format("d/m/Y");
            
            /* Se for retificação do evento utiliza a data de envio do original para considerar as mesmas situações da época do primeiro envio */
            $dataEnvioOrig = null;
            if ($parametros['IDOPERACAO'] == 'R') {
                $dataEnvioOrig = $parametros['DTRESPOSTAORIG'];
            }
            
            $dados = $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRESTRUTURAM'], $parametros['NRTPAMBIENTE'], $dataHistorico);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2205*/
SELECT VM.NRVINCULOM CHAVE,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideTrabalhador
       REPLACE(REPLACE(PH.NRCPFPESSOA, '.'), '-') AS CPFTRAB,
       --alteracao
       TO_CHAR(DECODE(PE.DTESOCIAL,TO_DATE(:P_DTHISTORICO),PE.DTESOCIAL,PH.DTULTATU), 'YYYY-MM-DD') AS DTALTERACAO,
       --dadosTrabalhador
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       PH.NMPESSOA AS NMTRAB,
       PH.IDSEXOPESSOA AS SEXO,
       RP.CDESOCIAL AS RACACOR,
       EC.CDESOCIAL AS ESTCIV,
       GI.CDESOCIAL AS GRAUINSTR,
       '' AS NMSOC,
       PB.CDESOCIAL paisNac,
        --endereco
        --brasil
        NVL(LG.CDESOCIAL,'O') AS TPLOGRAD,        
        EP.DSENDERECO AS DSCLOGRAD,
        NVL(EP.NRIMOVELENDERECO, '') AS NRLOGRAD,
        SUBSTR(EP.DSCOMPLEENDERECO, 0, 30) AS COMPLEMENTO,
        EP.NMBAIRROENDERECO AS BAIRRO,
        REPLACE(REPLACE(REPLACE(EP.NRCEPENDERECO,' '),'.'),'-') AS CEP,
        MP.CDMUNICIBGE AS BRASIL_CODMUNIC,
        EP.SGESTADO AS BRASIL_UF,
        --exterior
        PC.CDESOCIAL AS PAISRESID,
        EE.DSENDERECO AS EXTERIOR_DSCLOGRAD,
        NVL(EE.NRIMOVELENDERECO, '') AS EXTERIOR_NRLOGRAD,
        SUBSTR(EE.DSCOMPLEENDERECO, 0, 30) AS EXTERIOR_COMPLEMENTO,
        EE.NMBAIRROENDERECO AS EXTERIOR_BAIRRO,
        ME.NMMUNICIPIO AS NMCID,
        REPLACE(REPLACE(REPLACE(EE.NRCEPENDERECO,' '),'.'),'-') AS CODPOSTAL,
       --trabImig
       DECODE(PB.CDESOCIAL,'105','',DECODE(PH.IDPOSVISTOPERM,'S','2','1')) tmpResid,
       DECODE(PB.CDESOCIAL,'105','','7') condIng,
        --infoDeficiencia
        DECODE(PH.NRCONDFISPES,'2','S','N') AS DEFFISICA,        
        DECODE(PH.NRCONDFISPES,'4','S','N') AS DEFVISUAL,
        DECODE(PH.NRCONDFISPES,'3','S','N') AS DEFAUDITIVA,
        DECODE(PH.NRCONDFISPES,'5','S','N') AS DEFMENTAL,
        DECODE(PH.NRCONDFISPES,'8','S','N') AS DEFINTELECTUAL,
        DECODE(PH.NRCONDFISPES,'7','S','N') AS REABREADAP,        
        DECODE(VM.NRTIPOVINCULOM, 1, NVL(PH.IDCOTAPCD, 'N'), '') AS INFOCOTA,
        '' AS OBSERVACAO,
        --<<dependentes>>-- getDadosS2200dependente
       --contato
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '01') fonePrinc, -- 1-Telefone principal
       (SELECT MAX(C.CDCOMUNICAPARC)
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '05') emailPrinc, -- 5-email
       VM.NRORG, VM.NRVINCULOM, PN.NRPARCNEGOCIO
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*RaçaCor*/
  LEFT JOIN GPE_RACAPESSOA RP ON RP.NRRACAPESSOA = PH.NRRACAPESSOA
                             AND (RP.NRORG = PH.NRORG OR RP.NRORG = 0)
  /*Estado Civil*/
  LEFT JOIN ESTADOCIVIL EC ON EC.CDESTACIVIL = PH.CDESTACIVIL
                          AND (EC.NRORG = PH.NRORG OR EC.NRORG = 0)
  /*Grau Instrução*/
  LEFT JOIN GPE_GRAUINSTR GI ON GI.NRGRAUINSTR = PH.NRGRAUINSTR
                            AND (GI.NRORG = PH.NRORG OR GI.NRORG = 0)
  /*Dados do Empregado - Naturalidade*/
  LEFT JOIN GPE_NACIONALIDADE NA ON PH.NRNACIONALID = NA.NRNACIONALIDADE
                                AND (PH.NRORG = NA.NRORG OR NA.NRORG = 0)
  LEFT JOIN PAIS PB ON PB.CDPAIS = NA.CDPAIS
  /*Endereço Principal*/
  LEFT JOIN ENDERECOPARC EP ON EP.NRPARCNEGOCIO = PN.NRPARCNEGOCIO
                           AND EP.NRORG = PN.NRORG
                           AND EP.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MP ON MP.CDPAIS = EP.CDPAIS
                        AND MP.SGESTADO = EP.SGESTADO
                        AND MP.CDMUNICIPIO = EP.CDMUNICIPIO
  /*Endereço Exterior*/
  LEFT JOIN ENDERECOPARC EE ON EE.NRPARCNEGOCIO = PN.NRPARCNEGOCIO
                           AND EE.NRORG = PN.NRORG
                           AND EE.CDTIPOENDERECO = 'EXTERIOR'
  LEFT JOIN MUNICIPIO ME ON ME.CDPAIS = EE.CDPAIS
                        AND ME.SGESTADO = EE.SGESTADO
                        AND ME.CDMUNICIPIO = EE.CDMUNICIPIO
  LEFT JOIN PAIS PC ON PC.CDPAIS = EE.CDPAIS
  /*Logradouro*/
  LEFT JOIN LOGRADOURO LG ON LG.CDLOGRADOURO = EP.CDLOGRADOURO
                         AND (LG.NRORG = EP.NRORG OR LG.NRORG = 0) 
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM

  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getDadosS2205dependente($parametros) {
        try {
            $dados = $this->getDadosdependente($parametros['NRORG'], $parametros['NRPARCNEGOCIO'], $parametros['NRVINCULOM']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosdependente($nrorg, $nrparcnegocio, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2205dependente*/
SELECT LPAD(T.CDESOCIAL,2,'0') tpDep,
       H.NMPESSOA nmDep,
       TO_CHAR(H.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       REPLACE(REPLACE(H.NRCPFPESSOA, '.'), '-') cpfDep,
       H.IDSEXOPESSOA sexoDep,
       DECODE((SELECT NVL(COUNT(*),0) FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '2'),0,'N','S') depIRRF,
       DECODE((SELECT NVL(COUNT(*),0) FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '1'),0,'N','S') depSF,
       DECODE(NVL(H.NRCONDFISPES,1),1,'N','S') incTrab
  FROM RELACIONAPARC R
  JOIN TIPORELACIONA T ON (T.NRORG = R.NRORG OR T.NRORG = 0)
                      AND T.NRTIPORELACIONA = R.NRTIPORELACIONA
  JOIN GPE_PESSOA P ON R.NRPARCNEGRELAC = P.NRPARCNEGOCIO
                   AND R.NRORG = P.NRORG
                   AND EXISTS (SELECT 1 FROM FPA_DEPVINCULO D
                                WHERE D.NRPESSOA = P.NRPESSOA
                                  AND D.NRORG = P.NRORG
                                  AND D.NRVINCULOM = :P_NRVINCULOM)
  JOIN GPE_PESSOAH H ON P.NRPESSOA = H.NRPESSOA
                    AND P.NRORG = H.NRORG
                    AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                             WHERE H1.NRPESSOA = P.NRPESSOA
                                               AND H1.NRORG = P.NRORG
                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE R.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
   AND R.NRORG = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
            "P_NRVINCULOM" => $nrvinculom,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        );
        return $this->connection->fetchAll($sql, $params);
    }

}