<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\AlteOcupacaoRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class AlteOcupacaoService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, VinculoServices $vinculoServices, AlteOcupacaoRepository $alteOcupacaoRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->vinculoServices = $vinculoServices;
        $this->alteOcupacaoRepository = $alteOcupacaoRepository;

        $this->gpe_alteOcupacao = Repositories::GPE_ALTEOCUPACAO;
    }

    public function nova($nrorg,$nrvinculom, $nrOcupacao, $dtIniOcupacao, $nrTipoOcupacao, $nrTpModalidade,
    $nrMotivoOcupacao = null, $nrNivelOcupacao = null, $dtFimOcupacao = null, $observacao = null)
    {
        $alteOcupacao = new $this->gpe_alteOcupacao;

        $alteOcupacao->setNrorg($nrorg);
        $alteOcupacao->setNrvinculom($nrvinculom);
        $alteOcupacao->setNrocupacaom($nrOcupacao);
        $alteOcupacao->setDtiniocupacao($dtIniOcupacao);

        $alteOcupacao->setNrtipoocupacao($nrTipoOcupacao);

        $alteOcupacao->setNrmotivoocupa($nrMotivoOcupacao);
        $alteOcupacao->setNrnivelocupa($nrNivelOcupacao);
        $alteOcupacao->setNrtpmodalidsal($nrTpModalidade);
        $alteOcupacao->setDsobservacao($observacao);
        $alteOcupacao->setDtfimocupacao($dtFimOcupacao);

        $alteOcupacao->setDtinclusao(DateUtil::getDataAtual());
        $alteOcupacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteOcupacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteOcupacao->setDtultatu(DateUtil::getDataAtual());
        $alteOcupacao->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteOcupacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $alteOcupacao->setNralteocupacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEOCUPACAO', 1, 12, true));

        $this->entityManager->persist($alteOcupacao);
    }

    public function beforeSave($nrorg, $nrvinculom, $dtiniocupacao) {
        try {
            $this->validaExisteDataOcupacao($nrorg, $nrvinculom, $dtiniocupacao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function afterInsert($nrorg, $nrvinculom, $dtiniocupacao) {
        try {
            $dtiniocupacao = DateUtil::getDataDeString(substr($dtiniocupacao, 0, 10), DateUtil::FORMATO_BRASILEIRO);
            $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom, $dtiniocupacao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function afterSave($nrorg, $nrvinculom, $nrocupacaom, $dtiniocupacao, $dtfimocupacao, $idusarcomp = false) {
        try {
            $dataCompetencia = DateUtil::getDataDeString(substr(
                $this->hcmParameters->getDtCompetencia(),
                0,
                10),
                DateUtil::FORMATO_BRASILEIRO);
            $dataInicioOcupacao = DateUtil::getDataDeString(substr(
                $dtiniocupacao,
                0,
                10),
                DateUtil::FORMATO_BRASILEIRO);
            if ($dataInicioOcupacao > $dataCompetencia) {
                $dtmescompetenc = $dtiniocupacao;
            }
            if ($dataInicioOcupacao <= $dataCompetencia) {
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            }

            $dtiniproxocupacao = null;

            if($dtfimocupacao) $dtfimocupacao = DateUtil::getDataDeString(substr($dtfimocupacao, 0, 10), DateUtil::FORMATO_BRASILEIRO);

            if($idusarcomp){
                if ($dataInicioOcupacao > $dataCompetencia) {
                    $alteocupacao = $this->alteOcupacaoRepository->retornaUltimaAlteOcupacaoPorDataInicio($nrorg, $nrvinculom, DateUtil::getUltimoDiaDoMes($dtiniocupacao)->format(DateUtil::FORMATO_BRASILEIRO));
                }
                if ($dataInicioOcupacao <= $dataCompetencia) {
                    $alteocupacao = $this->alteOcupacaoRepository->retornaUltimaAlteOcupacaoPorDataInicio($nrorg, $nrvinculom, $this->hcmParameters->getUltimoDiaMes());
                }
                if(!empty($alteocupacao)) //Somente ajusta a ocupacao se existir uma
                    $nrocupacaom = $alteocupacao[0]['NROCUPACAOM'];
            }

            if ($dataInicioOcupacao > $dataCompetencia) {
                $proxalteocupacao = $this->alteOcupacaoRepository->retornaProximaAlteOcupacao($nrorg, $nrvinculom, DateUtil::getUltimoDiaDoMes($dtiniocupacao)->format(DateUtil::FORMATO_BRASILEIRO));
            }
            if ($dataInicioOcupacao <= $dataCompetencia) {
                $proxalteocupacao = $this->alteOcupacaoRepository->retornaProximaAlteOcupacao($nrorg, $nrvinculom, $this->hcmParameters->getUltimoDiaMes());
            }

            if ($proxalteocupacao) {
                $proxalteocupacao = $proxalteocupacao[0];
                $dtiniproxocupacao = DateUtil::getDataDeString(substr($proxalteocupacao['DTINIOCUPACAO'], 0, 10), DateUtil::FORMATO_BRASILEIRO);
                $dtiniproxocupacao->sub(new \DateInterval('P1D'));
                $dtfimocupacao = $dtiniproxocupacao;
            }

            $this->entityManager->beginTransaction();

            $this->atualizaOcupacaoHistoricoVinculo($nrorg, $nrvinculom, $nrocupacaom, $nrocupacaom, $dtmescompetenc);

            $this->entityManager->commit();

        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function afterDelete($nrorg, $nrvinculom, $dtiniocupacao) {
        $dataCompetencia = DateUtil::getDataDeString(substr(
            $this->hcmParameters->getDtCompetencia(),
            0,
            10),
            DateUtil::FORMATO_BRASILEIRO);
        $dataInicioOcupacao = DateUtil::getDataDeString(substr(
            $dtiniocupacao,
            0,
            10),
            DateUtil::FORMATO_BRASILEIRO);
        if ($dataInicioOcupacao <= $dataCompetencia) {
            $dtiniocupacao = $this->hcmParameters->getDtCompetencia();
        }

        $nrocupacaom = null;
        $alteocupacao = $this->alteOcupacaoRepository->retornaUltimaAlteOcupacaoPorDataInicio($nrorg, $nrvinculom, $dtiniocupacao);
        if (!empty($alteocupacao)) {
            $nrocupacaom = $alteocupacao[0]['NROCUPACAOM'];
        }

        $this->atualizaOcupacaoHistoricoVinculo($nrorg, $nrvinculom, $nrocupacaom, $nrocupacaom, $dtiniocupacao);
    }

    /**
     * Valida se já existe outra ocupacao para o vínculo com a mesma data de início
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $dtiniocupacao
     * @throws \Exception
     */
    public function validaExisteDataOcupacao($nrorg, $nrvinculom, $dtiniocupacao) {
        try {
            $alteocupacoes = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)
                ->findBy(array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom, 'dtiniocupacao' => DateUtil::getDataDeString($dtiniocupacao, DateUtil::FORMATO_BRASILEIRO, true)));
            if (!empty($alteocupacoes)) {
                throw new \Exception(Messages::HCM00000_GER_ALTEOCUPACAO_DATA_ALREADY_EXIST_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
            if(!empty($vinculom)) {
                $dtiniocupacao = DateUtil::getDataDeString($dtiniocupacao, DateUtil::FORMATO_BRASILEIRO, true);
                $dtadmissaovinc = DateUtil::getDataDeString($vinculom->getDtadmissaovinc(), DateUtil::FORMATO_BRASILEIRO, true);
                $maiorData = DateUtil::retornaMaiorData($dtiniocupacao, $dtadmissaovinc);
                if($maiorData == $dtadmissaovinc && $dtadmissaovinc->format('d/m/Y') !== $dtiniocupacao->format('d/m/Y')) {
                    throw new \Exception("Impossível adicionar uma movimentação anterior à data de admissão do vínculo!", Messages::ERROR_MESSAGE_CODE);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza número da ocupacao no histórico atual do vínculo
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $nrocupacaom
     * @throws \Exception
     */
    public function atualizaOcupacaoHistoricoVinculo($nrorg, $nrvinculom, $nrocupacaom, $nrfuncao = null, $dtmescompetenc = null) {
        try {
            $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc);
            $gpevinculoh->setNrcargo($nrocupacaom);
            $gpevinculoh->setNrfuncao($nrfuncao);
            //$gpevinculoh->setNrfaixa($nrfuncao);
            $gpevinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $gpevinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculoh->setDtultatu(DateUtil::getDataAtual());

            $this->entityManager->persist($gpevinculoh);
            $this->entityManager->flush($gpevinculoh);
        } catch (\Exception $e) {
            throw $e;
        }
    }

} 