<?php

namespace HCM\PTO\Logic\AFD\Portaria671;

/**
 * Portaria 671
 * Responsavel por gerar o conteudo do registro "Marcação de ponto para REP-P"
 * 
 */

use HCM\PTO\Logic\AFD\{ Parametros, RegistroPorConsulta };

class RegistroMarcacao extends RegistroPorConsulta {
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Parametros $parametros) {
        parent::__construct($entityManager, $parametros, new ValidadorRegistroSHA256);
    }
    
    /**
     * Portaria 671
     * Retorna a query a ser executada para a montagem dos itens do registro de marcacao
     * @return string
     */
    public function obtemConsultaRegistro(): string {
        return <<<SQL
            SELECT
                SUBSTR(
                    /* 001-009 NSR */ LPAD(MP.NRMARCACAOPONTO, 9, '0') ||
                    /* 010-010 Tipo do registro. Preencher com "7" */ '7' ||
                    /* 011-034 Data e hora da marcação de ponto */ TO_CHAR(MP.DTHRAJUSTADA, 'YYYY-MM-DD') || 'T' || TO_CHAR(MP.DTHRAJUSTADA, 'HH24:MI') || ':00' || TO_CHAR(FROM_TZ(CAST (MP.DTHRAJUSTADA AS TIMESTAMP), SESSIONTIMEZONE),'TZHTZM') ||
                    /* 035-046 CPF do empregado */ LPAD(NVL(REPLACE(TRANSLATE( PH.NRCPFPESSOA, TRANSLATE('0' || PH.NRCPFPESSOA, '0123456789', '@' ), '@' ), '@', '' ), '0'), 12, '0') ||
                    /* 047-070 Data e hora de gravação do registro */ TO_CHAR(MP.DTINCLUSAO, 'YYYY-MM-DD') || 'T' || TO_CHAR(MP.DTINCLUSAO, 'HH24:MI') || ':00' || TO_CHAR(FROM_TZ(CAST (MP.DTINCLUSAO AS TIMESTAMP), SESSIONTIMEZONE),'TZHTZM') ||
                    /* 071-072 Identificador do coletor da marcação: "01": aplicativo mobile; "02": browser (navegador internet); "03": aplicativo desktop; "04": dispositivo eletrônico; "05": outro dispositivo eletrônico não especificado acima. */ '01' ||
                    /* 073-073 Informar "0" para marcação on-line ou "1" para marcação off-line */ '1'
                    /* 074-137 Código hash. Gerado posteriormente */
                , 0, 73) AS AFD
            FROM GPE_MARCACAOPONTO MP
            LEFT JOIN PTO_PERIODOAPURACAO PA ON PA.NRORG = MP.NRORG
                AND PA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
            LEFT JOIN GPE_VINCULOM VM ON VM.NRORG = MP.NRORG AND VM.NRVINCULOM = MP.NRVINCULOM
            LEFT JOIN GPE_VINCULOH VH ON VH.NRORG = MP.NRORG AND VH.NRVINCULOM = MP.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                FROM GPE_VINCULOH
                WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM
            )
            LEFT JOIN GPE_PESSOAH PH ON PH.NRORG = MP.NRORG AND PH.NRPESSOA = VM.NRPESSOA AND PH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                FROM GPE_PESSOAH
                WHERE NRORG = PH.NRORG AND NRPESSOA = PH.NRPESSOA
            )
            WHERE MP.NRORG = :P_NRORG
                AND MP.NRCOLETOR NOT IN (21, 22, 25)
                AND (
                    CASE
                        WHEN MP.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                            MP.DTHRAJUSTADA - 1
                        WHEN MP.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                            MP.DTHRAJUSTADA + 1
                        ELSE MP.DTHRAJUSTADA 
                    END
                ) BETWEEN NVL(TO_DATE(:P_INICIO, 'DD/MM/YYYY'), PA.DTINICIALAPURACAO) AND NVL(TO_DATE(:P_FIM, 'DD/MM/YYYY'), PA.DTFINALAPURACAO)
                AND ( INSTR(',' || :NRVINCULOM_LIST || ',', ',' || VM.NRVINCULOM || ',') > 0 OR :NRVINCULOM_LIST IS NULL )
                AND ( INSTR(',' || :NRESTRUTLEGAL_LIST || ',', ',' || VH.NRESTRUTLEGAL || ',') > 0 OR :NRESTRUTLEGAL_LIST IS NULL )
                AND ( INSTR(',' || :NRESTRUTGEREN_LIST || ',', ',' || VH.NRESTRUTGEREN || ',') > 0 OR :NRESTRUTGEREN_LIST IS NULL )
                AND ( INSTR(',' || :NRTIPOVINCULOM_LIST || ',', ',' || VM.NRTIPOVINCULOM || ',') > 0 OR :NRTIPOVINCULOM_LIST IS NULL )
            ORDER BY MP.NRMARCACAOPONTO, MP.DTHRAJUSTADA
SQL;
    }
    
    /**
     * Portaria 671
     * Retorna os parametros da query a ser executada para a montagem dos itens do registro de marcacao
     * @return array
     */
    public function obtemParametrosConsultaRegistro(): array {
        return [
            'P_NRORG' => $this->getParametros()->getNrorg(),
            'P_NRPERIODOAPURACAO' => $this->getParametros()->getNrPeriodoApuracao(),
            'P_INICIO' => $this->getParametros()->getDataInicio(),
            'P_FIM' => $this->getParametros()->getDataFim(),
            'NRVINCULOM_LIST' => $this->getParametros()->getVinculosString(),
            'NRESTRUTLEGAL_LIST' => $this->getParametros()->getEstruturasLegaisString(),
            'NRESTRUTGEREN_LIST' => $this->getParametros()->getEstruturasGerenciaisString(),
            'NRTIPOVINCULOM_LIST' => $this->getParametros()->getTiposVinculoString(),
        ];
    }
}